#include "Status.h"
#include "Vram.h"
#include "Stage.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"
#include "CopyMemory.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + 1, "SCORE");
    PrintS(Vram + 7, "HI-SCORE");
    PrintS(Vram + 16, "STAGE");
    PrintByteNumber2(Vram + VramRowSize + 18, CurrentStage + 1);
    PrintS(Vram+ 22, "TIME");

    {
        ptr<byte> pVram;
        pVram = Vram + 27;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            do {
                byte c;
                c = Char_Man;
                repeat (2) {
                    repeat (2) {
                        *pVram = c;
                        ++pVram;
                        ++c;
                    }
                    pVram += VramRowSize - 2;
                }
                pVram += 2 - VramRowSize * 2;
                --i;
            } while (i > 0);
        }
    }

    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramRowSize, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramRowSize + 9, HiScore);
    PrintC(pVram, '0');
}

void PrintTime()
{
    PrintByteNumber3(Vram + VramRowSize + 23, StageTime);
}

static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    ptr<byte> pVram;

    pVram = Vram + VramRowSize * 11 + 10;
    repeat (height) {
        repeat (width) {
            pVram = PrintC(pVram, ' ');
        }
        pVram += VramRowSize - width;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 12 + 11, "GAME OVER");
}

void PrintTimeUp() 
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 12 + 12, "TIME UP");
}

void Title()
{
    static const byte[] TitleBytes = {
		0x4e, 0x6e, 0x66, 0x73, 
		0x4e, 0x54, 0x76, 0x78, 
		0x4e, 0x00, 0x68, 0x72, 
		0x6e, 0x6e, 0x6e, 0x00, 
		0x00, 0x00, 0x00, 0x00, 
		0x76, 0x66, 0x73, 0x76, 
		0x4e, 0x68, 0x72, 0x4e, 
		0x7a, 0x6e, 0x00, 0x7a, 
		0x00, 0x00, 0x61, 0x73, 
		0x66, 0x73, 0x66, 0x64, 
		0x68, 0x72, 0x68, 0x72, 
		0x6e, 0x00, 0x00, 0x6e, 
		0x00, 0x00, 0x00, 0x68, 
		0x61, 0x64, 0x6e, 0x68, 
		0x00, 0x6e, 0x63, 0x68, 
		0x7a, 0x6e, 0x78, 0x7a, 
		0x72, 0x00, 0x00, 0x00, 
		0x63, 0x64, 0x61, 0x64, 
		0x4e, 0x73, 0x68, 0x64, 
		0x78, 0x6e, 0x00, 0x6e, 
		0x00, 0x68, 0x72, 0x4e, 
		0x63, 0x68, 0x72, 0x4e, 
		0x6e, 0x68, 0x72, 0x4e, 
		0x78, 0x7a, 0x78, 0x6e, 
   };

    ClearScreen();
    CopyMemory(
        CharGen + (Char_Man - 0x80) * PatternSize, 
        Pattern_Man,
        PatternSize * 4
    );

    HideAllSprites();
    PrintStatus();
    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 7 + 4;
        p = TitleBytes;
        repeat (6) {
            repeat (4) {
                repeat (4) {
                    *pVram = *p;
                    ++pVram;
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 4;
        }
    }
    PrintS(Vram + VramRowSize * 19 + 9, "PUSH \"Z\" KEY");
    PrintS(Vram + VramRowSize * 20 + 9, "OR \"X\" KEY");
}